// compile: gcc -Wall VirtualFunctions-CSoln.c -o VirtualFunctions-CSoln

#include <stdio.h>


struct _Base;
void Base_print(const struct _Base *const this);

typedef struct _BaseVTable {
  void (*print)(const struct _Base *const this);
} BaseVTable;

typedef struct _Base {
  const BaseVTable *vtable;
  float x;
} Base;


void Base_init  (Base *const this, const float x) {
  static const BaseVTable basevtable = { Base_print };

  this->vtable = &basevtable;
  this->x = x;
}
void Base_deinit(Base *const this)                { }

void Base_print(const Base *const this) { printf("%g\n",this->x); }



struct _Derived;
void Derived_print(const struct _Derived *const this);

typedef struct _Derived {
  Base base;
  float y;
} Derived;


void Derived_init(Derived *const this, const float x,const float y) {
  static const BaseVTable derivedvtable = { (void (*)(const Base *const))Derived_print };

  Base_init(&this->base, x);
  this->base.vtable = &derivedvtable;
  this->y = y;
}
void Derived_deinit(Derived *const this) { Base_deinit(&this->base); }

void Derived_print(const Derived *const this) {
  Base_print(&this->base);
  printf("%g\n",this->y);
}



int main() {
  Derived x;  Derived_init(&x, 1.0,2.0);
  Base *y = &x.base;

  printf("Printing x:\n");
  x.base.vtable->print(&x.base);
  printf("Printing x through y:\n");
  y->vtable->print(y);

  return 0;
}
